#include "shape.h"
#include "window.h"

#include <QMimeData>
#include <QGraphicsSceneDragDropEvent>
#include <QPen>
#include <QBrush>
#include <QBitmap>
#include <QPainter>

/* Shape */

QGraphicsItem * createElement (QString type)
{
    QAbstractGraphicsShapeItem * shape = NULL;
    if (type == "rectangle")
        shape = new Rectangle;
    else if (type == "ellipse")
        shape = new Ellipse;
    return shape;
}

void shapeDragEnter (QGraphicsSceneDragDropEvent *event)
{
    const QMimeData * data = event->mimeData();
    if (data->hasColor() || data->hasFormat (shape_format))
        event->setAccepted (true);
}

void shapeDrop (QAbstractGraphicsShapeItem * shape, QGraphicsSceneDragDropEvent *event)
{
   const QMimeData * data = event->mimeData();
   if (data->hasColor())
   {
       QColor c =  data->colorData().value <QColor> ();
       if (event->proposedAction() == Qt::MoveAction) /* shift */
          shape->setPen (c);
       else
          shape->setBrush (c);
       refreshTree ();
   }
   if (data->hasFormat (shape_format))
   {
       QString code = data->data (shape_format);
       QGraphicsItem * item = createElement (code);
       QPointF p = event->scenePos();
       p = p - shape->scenePos();
       item->setPos (p);
       item->setParentItem(shape);
       refreshTree ();
   }
}

/* Rectangle */

void Rectangle::dragEnterEvent (QGraphicsSceneDragDropEvent *event)
{
    shapeDragEnter (event);
}

void Rectangle::dropEvent (QGraphicsSceneDragDropEvent *event)
{
    shapeDrop (this, event);
}

Rectangle::Rectangle()
{
    setAcceptDrops (true);
    setRect (0, 0, 200, 100);
    setPen (QColor ("blue"));
    setBrush (QColor ("orange"));
    setFlag (QGraphicsItem::ItemIsMovable);
    setFlag (QGraphicsItem::ItemIsSelectable);
}

/* Ellipse */

void Ellipse::dragEnterEvent (QGraphicsSceneDragDropEvent *event)
{
    shapeDragEnter (event);
}

void Ellipse::dropEvent (QGraphicsSceneDragDropEvent *event)
{
    shapeDrop (this, event);
}

Ellipse::Ellipse ()
{
    setAcceptDrops (true);
    setRect (0, 0, 200, 100);
    setPen (QColor ("blue"));
    setBrush (QColor ("yellow"));
    setFlag (QGraphicsItem::ItemIsMovable);
    setFlag (QGraphicsItem::ItemIsSelectable);
}

/* Plane */

void Plane::dragEnterEvent (QGraphicsSceneDragDropEvent *event)
{
    shapeDragEnter (event);
}

void Plane::dropEvent (QGraphicsSceneDragDropEvent *event)
{
    shapeDrop (this, event);
}

Plane::Plane ()
{
    setAcceptDrops (true);
    // setFlag (QGraphicsItem::ItemIsMovable);
    // setFlag (QGraphicsItem::ItemIsSelectable);

      const int n = 16;
      QBitmap texture (n, n);
      texture.clear ();

      QPainter painter (& texture);
      painter.drawLine (0, 0, n-1, 0);
      painter.drawLine (0, 0, 0, n-1);

      QBrush brush (QColor ("cornflowerblue"));
      brush.setTexture (texture);
      setBrush (brush);

      // setPen (QPen (Qt::NoPen));
      // setBrush (QBrush (Qt::NoBrush));
      // setFlags (0);
      setRect (0, 0, 1280, 1024);
}
